#!/usr/bin/env python
 
###############################################################################
#
# Module:
#    eqn_regr_test.py
#
# Purpose:
#    Given an input file of otherwise-verified equations and solutions,
#    this will run through each equation using eqn_test_driver and verify
#    that it generates the correct soltuion.
#
# Format:
#    The format of the equations and solutions file is like (excluding '#'s):
#
#----------------------------------------------------------------------------
# 1+1     = 2
# 7-4     = 3
# 54.6+2  = 56.6
#----------------------------------------------------------------------------
#
# History:
#
# Date       PR      Who       Why
# -------------------------------------------------------------------------
# 30May2006  57334   Sontag    First version
#
###############################################################################

import math, os, sys, time
import PipelineUtils


# simulate a capitalized CEIL and FLOOR
def CEIL(x):  return math.ceil(x)
def FLOOR(x): return math.floor(x)

def printUsageExit():
   me = os.path.basename(sys.argv[0])
   print "Usage:  "+ \
         me+" <equations-and-solutions-filename> [-verify] [-quiet]"
   sys.exit(1)


#
# main routine
#
if (__name__ == '__main__'): # in case something else imports this file

   # args check
   if (len(sys.argv)<2): printUsageExit()
   chatty = ("-quiet" not in sys.argv)

   # Read in the equations-and-solutions file, populate list<tuple<str,str>>
   lines = open(sys.argv[1]).readlines()
   eqnsSolns = []
   for line in lines:
      if (len(line.strip()) <= 0): continue # handle empty lines
      if (line.strip()[0] == '#'): continue # handle comments
      aPair = line.split("=")
      eqn = aPair[0].strip() # remove surrounding but not embedded whitespace
      sol = aPair[1].strip()
      eqnsSolns.append( (eqn,sol) ) # each item is a tuple

   # First verify each one via Python's eval feature, if asked to
   if ("-verify" in sys.argv):
      for item in eqnsSolns:
         if (chatty): print "Verifying via Python that: "+item[0]+" = "+item[1]
         jj = `eval(item[0])`
         if (jj != item[1]):
            # don't complain if the diff is "22.0" (Python) vs. "22" (eqn...)
            if (len(jj)>2 and jj[-2:]=='.0' and jj[:-2]==item[1]): continue
            # throw now?  Wait, lets first see if this is precision error
            a = float(eval(item[0]))
            b = float(item[1])
            x = abs(1.0 - abs(a/b))
            assert x < 1.0E-12, \
                   "Error, Python thinks "+item[0]+" = "+jj+", not "+item[1]
      print "Done the extra verification stage using Python's eval()\n\n"

   # Now call eqn_test_driver to test it on each one (single-quote the eqn)
   for item in eqnsSolns:
      cmd = "eqn_test_driver '"+item[0]+"'"
      # The following gets stdout and stderr, so we needn't check $status
      out = PipelineUtils.getOutputOf(cmd)
      # Here is where the rubber hits the road
      assert out.strip() == item[1], \
         "Error, eqn_test_driver thinks "+item[0]+" = "+out+", not "+item[1]
      # progress
      if (chatty):
         print "Testing eqn_test_driver: "+item[0]+" = "+item[1]+" ... OK"

   # Done (if we get here than all were successful)
   sys.exit(0)
